-- 11_Legal_Offers_Objects_Script
use	WideWorldImporters
;
go

exec	sp_configure
		'clr enabled'
		,1
;
reconfigure
;
go

-- Schema(s)	
if	(schema_id('Utility') is null)
	begin
		exec	(N'
			create schema	Utility
				authorization	dbo
			;
			')
	end
go

if	(schema_id('Offers') is null)
	begin
		exec	(N'
			create schema	Offers
				authorization	dbo
			;
			')
	end
go

-- Tables (DROP)
if	(object_id('Offers.SellerItems') is not null)
	begin
		drop table	Offers.SellerItems
	end
go

if	(object_id('Offers.Sellers') is not null)
	begin
		drop table	Offers.Sellers
	end
go

if	(object_id('Offers.BuyerItems') is not null)
	begin
		drop table	Offers.BuyerItems
	end
go

if	(object_id('Offers.Buyers') is not null)
	begin
		drop table	Offers.Buyers
	end
go

if	(object_id('Offers.SellerItemsMO') is not null)
	begin
		drop table	Offers.SellerItemsMO
	end
go

if	(object_id('Offers.SellersMO') is not null)
	begin
		drop table	Offers.SellersMO
	end
go

if	(object_id('Offers.BuyerItemsMO') is not null)
	begin
		drop table	Offers.BuyerItemsMO
	end
go

if	(object_id('Offers.BuyersMO') is not null)
	begin
		drop table	Offers.BuyersMO
	end
go

-- Functions (DROP)
if	(object_id('Utility.SystemValidUntil') is not null)
	begin
		drop function	Utility.SystemValidUntil
	end
go

if	(object_id('Utility.SystemValidSince') is not null)
	begin
		drop function	Utility.SystemValidSince
	end
go

-- Assembly
if	(exists (
		select	*
			from	sys.assemblies
			where	(assemblies.name = 'Utility')
		))
	begin
		drop assembly	Utility
	end
go

create assembly	Utility
from	N'C:\SQLCLR\Legal.Utility.SqlClr.dll'
	with	permission_set = safe
;
go

-- Functions (CREATE)
create function	Utility.SystemValidSince
	(
	@ValidSince				date							= null
	)
returns	datetime2(7)
as external name	Utility.[Legal.Utility.System].ValidSince
;
go

create function	Utility.SystemValidUntil
	(
	@ValidUntil				date							= null
	)
returns	datetime2(7)
as external name	Utility.[Legal.Utility.System].ValidUntil
;
go

-- Tables (CREATE)
create table	Offers.Sellers
	(
	OfferID					int					identity(1, 1)	not null
	,PersonID				int							not null
	,CreatedOn				datetime2(7)						not null
	,ValidSince				datetime2(7)						not null
	,ValidUntil				datetime2(7)						null
	,SystemValidSince
		as
		(
		Utility.SystemValidSince
			(
			ValidSince
			)
		)
		persisted
	,SystemValidUntil
		as
		(
		Utility.SystemValidUntil
			(
			ValidUntil
			)
		)
		persisted
	,constraint	fk_Offers_Sellers_Application_People_PersonID
			foreign key
			(
			PersonID
			)
			references	Application.People
			(
			PersonID
			)
	)
go

create table	Offers.SellerItems
	(
	OfferID					int							not null
	,StockItemID				int							not null
	,Quantity				int							not null
	,constraint	fk_Offers_SellerItems_Warehouse_StockItems_StockItemID
			foreign key
			(
			StockItemID
			)
			references	Warehouse.StockItems
			(
			StockItemID
			)
	)
go

create table	Offers.Buyers
	(
	OfferID					int					identity(1, 1)	not null
	,PersonID				int							not null
	,CreatedOn				datetime2(7)						not null
	,ValidSince				datetime2(7)						not null
	,ValidUntil				datetime2(7)						null
	,SystemValidSince
		as
		(
		Utility.SystemValidSince
			(
			ValidSince
			)
		)
		persisted
	,SystemValidUntil
		as
		(
		Utility.SystemValidUntil
			(
			ValidUntil
			)
		)
		persisted
	,constraint	fk_Offers_Buyers_Application_People_PersonID
			foreign key
			(
			PersonID
			)
			references	Application.People
			(
			PersonID
			)
	)
go

create table	Offers.BuyerItems
	(
	OfferID					int							not null
	,StockItemID				int							not null
	,Quantity				int							not null
	,constraint	fk_Offers_BuyerItems_Warehouse_StockItems_StockItemID
			foreign key
			(
			StockItemID
			)
			references	Warehouse.StockItems
			(
			StockItemID
			)
	)
go

create table	Offers.SellersMO
	(
	OfferID					int					identity(1, 1)	not null
	,PersonID				int							not null
	,CreatedOn				datetime2(7)						not null
	,ValidSince				datetime2(7)						not null
	,ValidUntil				datetime2(7)						null
	,constraint	pk_Offers_SellersMO
			primary key nonclustered
			(
			OfferID
			)
	,index	cx_Offers_SellersMO
		clustered columnstore
	)
	with	(
		memory_optimized = on
		,durability = schema_and_data
		)
go

create table	Offers.SellerItemsMO
	(
	OfferID					int							not null
	,StockItemID				int							not null
	,Quantity				int							not null
	,constraint	pk_Offers_SellerItemsMO
			primary key nonclustered
			(
			StockItemID
			,OfferID
			)
	,index	cx_Offers_SellerItemsMO
		clustered columnstore
	)
	with	(
		memory_optimized = on
		,durability = schema_and_data
		)
go

create table	Offers.BuyersMO
	(
	OfferID					int					identity(1, 1)	not null
	,PersonID				int							not null
	,CreatedOn				datetime2(7)						not null
	,ValidSince				datetime2(7)						not null
	,ValidUntil				datetime2(7)						null
	,constraint	pk_Offers_BuyersMO
			primary key nonclustered
			(
			OfferID
			)
	,index	cx_Offers_BuyersMO
		clustered columnstore
	)
	with	(
		memory_optimized = on
		,durability = schema_and_data
		)
go

create table	Offers.BuyerItemsMO
	(
	OfferID					int							not null
	,StockItemID				int							not null
	,Quantity				int							not null
	,constraint	pk_Offers_BuyerItemsMO
			primary key nonclustered
			(
			StockItemID
			,OfferID
			)
	,index	cx_Offers_BuyItemsMO
		clustered columnstore
	)
	with	(
		memory_optimized = on
		,durability = schema_and_data
		)
go
