-- 11c_Legal_Offers_Procedures_Script
if	(object_id('Offers.AddNewMatches') is not null)
	begin
		drop proc	Offers.AddNewMatches
	end
go

create proc	Offers.AddNewMatches
	(
	@Divisor				int
	,@QuantityIncrease			int
	,@RowCountRestriction			int							= null
	)
as
begin
	declare	@now				datetime2(7)

	create table	#newBuyers
		(
		NewBuyerID			int					identity(1, 1)	not null
		,PersonID			int							not null
		,CreatedOn			datetime2(7)						not null
		,ValidSince			datetime2(7)						not null
		,ValidUntil			datetime2(7)						null
		,StockItemID			int							not null
		,Quantity			int							not null
		,primary key clustered
			(
			NewBuyerID
			)
		,unique nonclustered
			(
			PersonID
			,CreatedOn
			,StockItemID
			,NewBuyerID
			)
		)

	set	@QuantityIncrease = coalesce(@QuantityIncrease, 0)
	set	@RowCountRestriction = coalesce(@RowCountRestriction, 100)
	set	@now = getdate()

	begin
		with	Sellers
		as
		(
		select	top(@RowCountRestriction)
			max(Sellers.OfferID) as OfferID
			,PersonID
				= (
				select	top(1)
					People.PersonID as PersonID
					from	Application.People
					where	(People.PersonID > 1)
						and (People.PersonID != Sellers.PersonID)
					order by	newid()
				)
			,max(Sellers.ValidSince) as ValidSince
			,max(Sellers.ValidUntil) as ValidUntil
			from	Offers.Sellers
			where	(Sellers.PersonID % @Divisor = 0)
			group by	Sellers.PersonID
			order by	newid()
		)
		insert	#newBuyers
			(
			PersonID
			,CreatedOn
			,ValidSince
			,ValidUntil
			,StockItemID
			,Quantity
			)
			select	Sellers.PersonID
				,@now as CreatedOn
				,Sellers.ValidSince
				,Sellers.ValidUntil
				,SellerItems.StockItemID
				,Quantity
					= max(SellerItems.Quantity
					+ @QuantityIncrease)
				from	Sellers
					inner join	Offers.SellerItems
							on	SellerItems.OfferID = Sellers.OfferID
				group by	Sellers.PersonID
						,Sellers.ValidSince
						,Sellers.ValidUntil
						,SellerItems.StockItemID
	end

	insert	Offers.Buyers
		(
		PersonID
		,CreatedOn
		,ValidSince
		,ValidUntil
		)
		select	NewBuyers.PersonID
			,max(NewBuyers.CreatedOn) as CreatedOn
			,max(NewBuyers.ValidSince) as ValidSince
			,max(NewBuyers.ValidUntil) as ValidUntil
			from	#newBuyers NewBuyers
			where	(not exists (
					select	*
						from	Offers.Buyers
						where	(Buyers.PersonID = NewBuyers.PersonID)
							and (Buyers.CreatedOn = NewBuyers.CreatedOn)
					))
			group by	NewBuyers.PersonID

	insert	Offers.BuyerItems
		(
		OfferID
		,StockItemID
		,Quantity
		)
		select	Buyers.OfferID
			,NewBuyers.StockItemID
			,max(NewBuyers.Quantity) as Quantity
			from	Offers.Buyers
				inner join	#newBuyers NewBuyers
						on	(NewBuyers.PersonID = Buyers.PersonID)
							and (NewBuyers.CreatedOn = Buyers.CreatedOn)
			where	(not exists (
					select	*
						from	Offers.BuyerItems
						where	(BuyerItems.OfferID = Buyers.OfferID)
							and (BuyerItems.StockItemID = NewBuyers.StockItemID)
					))
			group by	Buyers.OfferID
					,NewBuyers.StockItemID

	insert	Offers.BuyersMO
		(
		PersonID
		,CreatedOn
		,ValidSince
		,ValidUntil
		)
		select	NewBuyers.PersonID
			,max(NewBuyers.CreatedOn) as CreatedOn
			,max(NewBuyers.ValidSince) as ValidSince
			,max(NewBuyers.ValidUntil) as ValidUntil
			from	#newBuyers NewBuyers
			where	(not exists (
					select	*
						from	Offers.BuyersMO
						where	(BuyersMO.PersonID = NewBuyers.PersonID)
							and (BuyersMO.CreatedOn = NewBuyers.CreatedOn)
					))
			group by	NewBuyers.PersonID

	insert	Offers.BuyerItemsMO
		(
		OfferID
		,StockItemID
		,Quantity
		)
		select	BuyersMO.OfferID
			,NewBuyers.StockItemID
			,max(NewBuyers.Quantity) as Quantity
			from	Offers.BuyersMO
				inner join	#newBuyers NewBuyers
						on	(NewBuyers.PersonID = BuyersMO.PersonID)
							and (NewBuyers.CreatedOn = BuyersMO.CreatedOn)
			where	(not exists (
					select	*
						from	Offers.BuyerItemsMO
						where	(BuyerItemsMO.OfferID = BuyersMO.OfferID)
							and (BuyerItemsMO.StockItemID = NewBuyers.StockItemID)
					))
			group by	BuyersMO.OfferID
					,NewBuyers.StockItemID

	drop table	#newBuyers

	return	0
end
go
