-- 12_Legal_Offers_Data_Script
use	WideWorldImporters
;
go

create table	#sellerItems
	(
	PersonID				int							not null
	,StockItemID				int							not null
	,Quantity				int							not null
	,CreatedOn				datetime2(7)						not null
		default
		(
		cast(dateadd(day, rand() * 12, getdate()) as datetime2(7))
		)
	,primary key clustered
		(
		PersonID
		,CreatedOn
		,StockItemID
		)
	)
go

declare	@peopleGroupSize			int
declare	@itemGroupSize				int

set	@peopleGroupSize = cast(rand() * 8 as int)
set	@itemGroupSize = cast(rand() * 6 as int)

set	@itemGroupSize
		= case
			when	(@itemGroupSize < 3)
				then	@itemGroupSize * 3
			when	(@itemGroupSize > 10)
				then	@itemGroupSize / 3
			else	@itemGroupSize
			end

begin
	with	People
	as
	(
	select	top(@peopleGroupSize)
		People.PersonID as PersonID
		from	Application.People
		where	(People.PersonID > 1)
		order by	newid()
	)
	insert	#sellerItems
		(
		PersonID
		,StockItemID
		,Quantity
		)
		select	People.PersonID as PersonID
			,StockItems.StockItemID as StockItemID
			,cast(cast(newid() as varbinary(2)) as int) / 100 * 10 as Quantity
			from	People
				cross apply	(
						select	top(@itemGroupSize)
							StockItems.StockItemID as StockItemID
							from	Warehouse.StockItems
							order by	newid()
						) StockItems
end

declare	@delay					datetime
set	@delay = '00:00:00.003'
waitfor delay	@delay
go 3000	-- This number determines the number of times the batch is run, and affects the number of cases.

truncate table	Offers.Sellers

insert	Offers.Sellers
	(
	PersonID
	,CreatedOn
	,ValidSince
	)
	select	distinct
		SourceSellerItems.PersonID as PersonID
		,SourceSellerItems.CreatedOn as CreatedOn
		,SourceSellerItems.CreatedOn as ValidSince
		from	#sellerItems SourceSellerItems

truncate table	Offers.SellerItems

insert	Offers.SellerItems
	(
	OfferID
	,StockItemID
	,Quantity
	)
	select	Sellers.OfferID as OfferID
		,SourceSellerItems.StockItemID as StockItemID
		,sum(SourceSellerItems.Quantity) as Quantity
		from	Offers.Sellers
			inner join	#sellerItems SourceSellerItems
					on	(SourceSellerItems.PersonID = Sellers.PersonID)
						and (SourceSellerItems.CreatedOn = Sellers.CreatedOn)
		group by	Sellers.OfferID
				,SourceSellerItems.StockItemID
go

delete	Offers.SellersMO

insert	Offers.SellersMO
	(
	PersonID
	,CreatedOn
	,ValidSince
	)
	select	distinct
		SourceSellerItems.PersonID as PersonID
		,SourceSellerItems.CreatedOn as CreatedOn
		,SourceSellerItems.CreatedOn as ValidSince
		from	#sellerItems SourceSellerItems

delete	Offers.SellerItemsMO

insert	Offers.SellerItemsMO
	(
	OfferID
	,StockItemID
	,Quantity
	)
	select	SellersMO.OfferID as OfferID
		,SourceSellerItems.StockItemID as StockItemID
		,sum(SourceSellerItems.Quantity) as Quantity
		from	Offers.SellersMO
			inner join	#sellerItems SourceSellerItems
					on	(SourceSellerItems.PersonID = SellersMO.PersonID)
						and (SourceSellerItems.CreatedOn = SellersMO.CreatedOn)
		group by	SellersMO.OfferID
				,SourceSellerItems.StockItemID
go

drop table	#sellerItems
go

create table	#buyerItems
	(
	PersonID				int							not null
	,StockItemID				int							not null
	,Quantity				int							not null
	,CreatedOn				datetime2(7)						not null
		default
		(
		cast(dateadd(day, rand() * 12, getdate()) as datetime2(7))
		)
	,primary key clustered
		(
		PersonID
		,CreatedOn
		,StockItemID
		)
	)
go

declare	@peopleGroupSize			int
declare	@itemGroupSize				int

set	@peopleGroupSize = cast(rand() * 8 as int)
set	@itemGroupSize = cast(rand() * 6 as int)

set	@itemGroupSize
		= case
			when	(@itemGroupSize < 3)
				then	@itemGroupSize * 3
			when	(@itemGroupSize > 10)
				then	@itemGroupSize / 3
			else	@itemGroupSize
			end

begin
	with	People
	as
	(
	select	top(@peopleGroupSize)
		People.PersonID as PersonID
		from	Application.People
		where	(People.PersonID > 1)
		order by	newid()
	)
	insert	#buyerItems
		(
		PersonID
		,StockItemID
		,Quantity
		)
		select	People.PersonID as PersonID
			,StockItems.StockItemID as StockItemID
			,cast(cast(newid() as varbinary(2)) as int) / 100 * 10 as Quantity
			from	People
				cross apply	(
						select	top(@itemGroupSize)
							StockItems.StockItemID as StockItemID
							from	Warehouse.StockItems
							order by	newid()
						) StockItems
end

declare	@delay					datetime
set	@delay = '00:00:00.003'
waitfor delay	@delay
go 3000	-- This number determines the number of times the batch is run, and affects the number of cases.

truncate table	Offers.Buyers

insert	Offers.Buyers
	(
	PersonID
	,CreatedOn
	,ValidSince
	)
	select	distinct
		SourceBuyerItems.PersonID as PersonID
		,SourceBuyerItems.CreatedOn as CreatedOn
		,SourceBuyerItems.CreatedOn as ValidSince
		from	#buyerItems SourceBuyerItems

truncate table	Offers.BuyerItems

insert	Offers.BuyerItems
	(
	OfferID
	,StockItemID
	,Quantity
	)
	select	Buyers.OfferID as OfferID
		,SourceBuyerItems.StockItemID as StockItemID
		,sum(SourceBuyerItems.Quantity) as Quantity
		from	Offers.Buyers
			inner join	#buyerItems SourceBuyerItems
					on	(SourceBuyerItems.PersonID = Buyers.PersonID)
						and (SourceBuyerItems.CreatedOn = Buyers.CreatedOn)
		group by	Buyers.OfferID
				,SourceBuyerItems.StockItemID
go

delete	Offers.BuyersMO

insert	Offers.BuyersMO
	(
	PersonID
	,CreatedOn
	,ValidSince
	)
	select	distinct
		SourceBuyerItems.PersonID as PersonID
		,SourceBuyerItems.CreatedOn as CreatedOn
		,SourceBuyerItems.CreatedOn as ValidSince
		from	#buyerItems SourceBuyerItems

delete	Offers.BuyerItemsMO

insert	Offers.BuyerItemsMO
	(
	OfferID
	,StockItemID
	,Quantity
	)
	select	BuyersMO.OfferID as OfferID
		,SourceBuyerItems.StockItemID as StockItemID
		,sum(SourceBuyerItems.Quantity) as Quantity
		from	Offers.BuyersMO
			inner join	#buyerItems SourceBuyerItems
					on	(SourceBuyerItems.PersonID = BuyersMO.PersonID)
						and (SourceBuyerItems.CreatedOn = BuyersMO.CreatedOn)
		group by	BuyersMO.OfferID
				,SourceBuyerItems.StockItemID
go

drop table	#buyerItems
go
