-- 14_Legal_Offers_Demo_Script
use	WideWorldImporters
;
go

set	statistics io, time	on
;
go

/*
select	count(*) as SellerCount
	from	Offers.Sellers

select	count(*) as SellerItemCount
	from	Offers.SellerItems

select	count(*) as SellerCountMO
	from	Offers.SellersMO

select	count(*) as SellerItemCountMO
	from	Offers.SellerItemsMO

select	count(*) as BuyerCount
	from	Offers.Buyers

select	count(*) as BuyerItemCount
	from	Offers.BuyerItems

select	count(*) as BuyerCountMO
	from	Offers.BuyersMO

select	count(*) as BuyerItemCountMO
	from	Offers.BuyerItemsMO
*/

-- Complete query (relational tables)
begin
	with	SellerItems
	as
	(
	select	Sellers.OfferID
		,Sellers.PersonID
		,Sellers.CreatedOn
		,Sellers.ValidSince
		,Sellers.ValidUntil
		,Sellers.SystemValidSince
		,Sellers.SystemValidUntil
		,SellerItems.StockItemID
		,SellerItems.Quantity
		from	Offers.Sellers
			inner join	Offers.SellerItems
					on	SellerItems.OfferID = Sellers.OfferID
	)
	,BuyerItems
	as
	(
	select	Buyers.OfferID
		,Buyers.PersonID
		,Buyers.CreatedOn
		,Buyers.ValidSince
		,Buyers.ValidUntil
		,Buyers.SystemValidSince
		,Buyers.SystemValidUntil
		,BuyerItems.StockItemID
		,BuyerItems.Quantity
		from	Offers.Buyers
			inner join	Offers.BuyerItems
					on	BuyerItems.OfferID = Buyers.OfferID
	)
	,CorrelatedOffers
	as
	(
	select	BuyerItems.OfferID as BuyerOfferID
		,SellerItems.OfferID as SellerOfferID
		from	SellerItems
			inner join	BuyerItems
					on	BuyerItems.StockItemID = SellerItems.StockItemID
		where	(1 = 1)
			and (SellerItems.PersonID != BuyerItems.PersonID)			-- Seller and buyer are not the same person.
			--and (SellerItems.Quantity = BuyerItems.Quantity)			-- Item quantities match.
			and (SellerItems.SystemValidSince <= BuyerItems.SystemValidUntil)	-- Offer and counter-offer are valid during
			and (BuyerItems.SystemValidSince <= SellerItems.SystemValidUntil)	-- the same time interval.
		group by	SellerItems.OfferID
				,BuyerItems.OfferID
		having	(count(*)
				= (
				select	count(*)
					from	Offers.BuyerItems MatchingBuyerItems
					where	(MatchingBuyerItems.OfferID = BuyerItems.OfferID)
				))
			and (count(*)
				= (
				select	count(*)
					from	Offers.SellerItems MatchingSellerItems
					where	(MatchingSellerItems.OfferID = SellerItems.OfferID)
				))
	)
	select	*
		from	CorrelatedOffers
		order by	CorrelatedOffers.SellerOfferID
				,CorrelatedOffers.BuyerOfferID
		--option	(
		--	maxdop	1
		--	)
end
go

-- Complete query (memory-optimized tables)
begin
	with	SellerItems
	as
	(
	select	Sellers.OfferID
		,Sellers.PersonID
		,Sellers.CreatedOn
		,Sellers.ValidSince
		,Sellers.ValidUntil
		,coalesce(Sellers.ValidSince, '00010101 00:00:00.0000000') as SystemValidSince
		,coalesce(Sellers.ValidUntil, '99991231 23:59:59.9999999') as SystemValidUntil
		,SellerItems.StockItemID
		,SellerItems.Quantity
		from	Offers.SellersMO Sellers
			inner join	Offers.SellerItemsMO SellerItems
					on	SellerItems.OfferID = Sellers.OfferID
	)
	,BuyerItems
	as
	(
	select	Buyers.OfferID
		,Buyers.PersonID
		,Buyers.CreatedOn
		,Buyers.ValidSince
		,Buyers.ValidUntil
		,coalesce(Buyers.ValidSince, '00010101 00:00:00.0000000') as SystemValidSince
		,coalesce(Buyers.ValidUntil, '99991231 23:59:59.9999999')  as SystemValidUntil
		,BuyerItems.StockItemID
		,BuyerItems.Quantity
		from	Offers.BuyersMO Buyers
			inner join	Offers.BuyerItemsMO BuyerItems
					on	BuyerItems.OfferID = Buyers.OfferID
	)
	,CorrelatedOffers
	as
	(
	select	BuyerItems.OfferID as BuyerOfferID
		,SellerItems.OfferID as SellerOfferID
		from	SellerItems
			inner join	BuyerItems
					on	BuyerItems.StockItemID = SellerItems.StockItemID
		where	(1 = 1)
			and (SellerItems.PersonID != BuyerItems.PersonID)			-- Seller and buyer are not the same person.
			--and (SellerItems.Quantity = BuyerItems.Quantity)			-- Item quantities match.
			and (SellerItems.SystemValidSince <= BuyerItems.SystemValidUntil)	-- Offer and counter-offer are valid during
			and (BuyerItems.SystemValidSince <= SellerItems.SystemValidUntil)	-- the same time interval.
		group by	SellerItems.OfferID
				,BuyerItems.OfferID
		having	(count(*)
				= (
				select	count(*)
					from	Offers.BuyerItemsMO MatchingBuyerItems
					where	(MatchingBuyerItems.OfferID = BuyerItems.OfferID)
				))
			and (count(*)
				= (
				select	count(*)
					from	Offers.SellerItemsMO MatchingSellerItems
					where	(MatchingSellerItems.OfferID = SellerItems.OfferID)
				))
	)
	select	*
		from	CorrelatedOffers
		order by	CorrelatedOffers.SellerOfferID
				,CorrelatedOffers.BuyerOfferID
		--option	(
		--	maxdop	1
		--	)
end
go

set	statistics io, time	off
;
go

--dbcc freeproccache
--dbcc dropcleanbuffers
--go
